@@ -3,14 +3,18 @@ import os
 import torch
 from torch.nn import CrossEntropyLoss
 import torch.optim
+from torch import optim
 import numpy as np
 from .config import ConfigBase
 from wcf.networks.utils import load_model
 import logging
 import json
 import time
-time_stamp=time.strftime('[%m%d]',time.localtime(time.time()))
-def load_weights(model, weights_path=None, except_keys=[],strict=False):
+
+time_stamp = time.strftime('[%m%d]', time.localtime(time.time()))
+
+
+def load_weights(model, weights_path=None, except_keys=[], strict=False):
     if weights_path:
         if os.path.exists(weights_path):
             try:
@@ -24,17 +28,18 @@ def load_weights(model, weights_path=None, except_keys=[],strict=False):
                     del state_dict[k]
                     print('ignore key:', k)
                 model.load_state_dict(state_dict, strict=strict)
-                print('Weights loaded from %s'%(weights_path))
+                print('Weights loaded from %s' % (weights_path))
             except:
                 logging.warning('Cannot load pretrained model %s' % (weights_path))
 
         if not os.path.exists(weights_path):
-            logging.warning('weights path %s does not exists.'%(weights_path))
+            logging.warning('weights path %s does not exists.' % (weights_path))
     return model
 
+
 class TrainValConfigBase(ConfigBase):
-    MODEL_TYPE='resnet18'
-    TIME_STAMP=time_stamp
+    MODEL_TYPE = 'resnet18'
+    TIME_STAMP = time_stamp
     PROJECT_TAG = ''
     EXTRA_TAG = ''
     TAG = None
@@ -44,8 +49,8 @@ class TrainValConfigBase(ConfigBase):
     DATA_DIR = None
     INPUT_SIZE = (224, 224)
     BATCH_SIZE = 16
-    BATCH_SIZE_TRAIN=None
-    BATCH_SIZE_VAL=None
+    BATCH_SIZE_TRAIN = None
+    BATCH_SIZE_VAL = None
     MAX_EPOCHS = 100
     PATIENCE = 20
     LR_INIT = 1e-3
@@ -63,47 +68,59 @@ class TrainValConfigBase(ConfigBase):
     VAL_INTERVAL = 1
     train_transform = None
     val_transform = None
-    USE_tqdm_TRAIN=True
-    USE_tqdm_VAL=False
-    GEN_CLASSES_FILE=False
-    CLASSES_FILE_PATH='classes.txt'
-    def get_TAG(self):
-        return '%s%s%s%s' % (self.PROJECT_TAG or '', self.TIME_STAMP or '', '[%s]'%(self.MODEL_TYPE), self.EXTRA_TAG or '')
+    USE_tqdm_TRAIN = True
+    USE_tqdm_VAL = False
+    GEN_CLASSES_FILE = False
+    CLASSES_FILE_PATH = 'classes.txt'
 
-    def __init__(self,**kwargs):
+    def get_TAG(self):
+        return '%s%s%s%s' % (
+        self.PROJECT_TAG or '', self.TIME_STAMP or '', '[%s]' % (self.MODEL_TYPE), self.EXTRA_TAG or '')
+
+    def get_train_data(self):
+        return Dataset(path=self.TRAIN_DIR, balance_classes=self.BALANCE_CLASSES, batch_size=self.BATCH_SIZE_TRAIN,
+                        transform=self.train_transform)
+
+    def get_val_data(self):
+        return Dataset(path=self.VAL_DIR, balance_classes=self.BALANCE_CLASSES_VAL, batch_size=self.BATCH_SIZE_VAL,
+                        transform=self.val_transform, shuffle=False)
+    def get_classes(self):
+        return sorted(os.listdir(self.TRAIN_DIR))
+    def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.BATCH_SIZE_TRAIN=self.BATCH_SIZE_TRAIN or self.BATCH_SIZE
-        self.BATCH_SIZE_VAL=self.BATCH_SIZE_VAL or self.BATCH_SIZE
+        self.BATCH_SIZE_TRAIN = self.BATCH_SIZE_TRAIN or self.BATCH_SIZE
+        self.BATCH_SIZE_VAL = self.BATCH_SIZE_VAL or self.BATCH_SIZE
         if self.TAG is None:
-            self.TAG =self.get_TAG()
-        self.train_data = Dataset(path=self.TRAIN_DIR, balance_classes=self.BALANCE_CLASSES, batch_size=self.BATCH_SIZE_TRAIN,
-                                  device=self.DEVICE, transform=self.train_transform)
-        self.val_data = Dataset(path=self.VAL_DIR, balance_classes=self.BALANCE_CLASSES_VAL, batch_size=self.BATCH_SIZE_VAL,
-                                device=self.DEVICE, transform=self.val_transform,shuffle=False)
+            self.TAG = self.get_TAG()
+        self.train_data = self.get_train_data()
+        self.val_data = self.get_val_data()
         if self.__class__.NUM_CLASSES is None:
-            self.__class__.NUM_CLASSES=self.train_data.num_classes
+            self.__class__.NUM_CLASSES = len(self.get_classes())
         self.model = self.get_model()
-        self.model=load_weights(self.model,weights_path=self.WEIGHTS_INIT)
+        self.model = load_weights(self.model, weights_path=self.WEIGHTS_INIT)
         self.model.to(self.DEVICE)
         if self.GEN_CLASSES_FILE:
-            with open(self.CLASSES_FILE_PATH,'w') as f:
-                f.write('\n'.join(self.train_data.classes))
-        print('CONFIG INFO'.center(200,'*'))
-        dic=self.get_config_info_dict()
+            with open(self.CLASSES_FILE_PATH, 'w') as f:
+                f.write('\n'.join(self.get_classes()))
+        print('CONFIG INFO'.center(200, '*'))
+        dic = self.get_config_info_dict()
         # max_length=max([len(k) for k in dic.keys()])
-        max_length=50
+        max_length = 50
         # print(max_length)
-        for k,v in dic.items():
+        for k, v in dic.items():
             # k=''
-            print('%s\t:\t%s'%(k.rjust(max_length,'_'),v))
+            print('%s\t:\t%s' % (k.rjust(max_length, '_'), v))
+
     def check_params(self):
         assert self.DATA_DIR or self.TRAIN_DIR
+
     def get_model(self, num_classes=None):
-        num_classes=num_classes or self.NUM_CLASSES
+        num_classes = num_classes or self.NUM_CLASSES
         assert num_classes
-        model=load_model(self.MODEL_TYPE,num_classes=num_classes,pretrained=self.USE_PRETRAINED)
+        model = load_model(self.MODEL_TYPE, num_classes=num_classes, pretrained=self.USE_PRETRAINED)
         return model
 
+
 class AccuracyMetric:
     def __init__(self):
         self.label_counts = 0
@@ -116,10 +133,11 @@ class AccuracyMetric:
             '''to prevent some arrays from being divided by zeros'''
             mask = t == 0
             epsilon = 1e-7
-            tmp=np.zeros_like(mask)
+            tmp = np.zeros_like(mask)
             tmp.fill(epsilon)
-            t = t +  tmp* mask
+            t = t + tmp * mask
             return t
+
         if self.sample_counts == 0:
             raise Exception('No samples for training?')
         else:
@@ -136,14 +154,17 @@ class AccuracyMetric:
     def batch_step(self, preds, labels):
         preds, labels = preds.cpu(), labels.cpu()
         batch_size, num_classes = preds.shape
-        torch.softmax(preds,1)
         _, preds = torch.max(preds, 1)
         labels = torch.zeros((batch_size, num_classes)).scatter_(-1, torch.unsqueeze(labels, -1), 1)
         preds = torch.zeros((batch_size, num_classes)).scatter_(-1, torch.unsqueeze(preds, -1), 1)
-        self.label_counts += torch.sum(labels, 0).numpy()
-        self.pred_counts += torch.sum(preds, 0).numpy()
-        self.correct_counts += torch.sum(labels * preds, 0).numpy()
-        self.sample_counts += len(labels)
+        l=torch.sum(labels, 0).numpy()
+        p=torch.sum(preds, 0).numpy()
+        c=torch.sum(labels * preds, 0).numpy()
+        s=len(labels)
+        self.label_counts += l
+        self.pred_counts += p
+        self.correct_counts += c
+        self.sample_counts += s
         # print(self.label_counts,self.pred_counts,self.correct_counts,self.sample_counts)
 
 
@@ -213,11 +234,13 @@ def val(cfg):
     model.eval()
     losses = []
     val_acc_metric = AccuracyMetric()
-    val_data=cfg.val_data
+    val_data = cfg.val_data
     if cfg.USE_tqdm_VAL:
         import tqdm
-        val_data=tqdm.tqdm(val_data)
+        val_data = tqdm.tqdm(val_data)
     for step, (inputs, labels) in enumerate(val_data):
+        inputs = inputs.to(cfg.DEVICE)
+        labels = labels.to(cfg.DEVICE)
         outputs = model(inputs)
         loss = cfg.criterion(outputs, labels)
         losses.append(loss.item())
@@ -227,13 +250,12 @@ def val(cfg):
     return res
 
 
-
 def train(cfg):
     assert isinstance(cfg, TrainValConfigBase)
     train_data = cfg.train_data
     model = cfg.model
-
     optimizer = torch.optim.Adam(model.parameters(), lr=cfg.LR_INIT)
+    # optimizer = optim.SGD(model.parameters(), lr=cfg.LR_INIT, momentum=0.9)
     lr_scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optimizer, cfg.MAX_EPOCHS, cfg.LR_END)
 
     global_step = -1
@@ -249,14 +271,16 @@ def train(cfg):
         train_acc_metric = AccuracyMetric()
         if cfg.USE_tqdm_TRAIN:
             import tqdm
-            train_data=tqdm.tqdm(train_data)
-        log='\n'+('Epoch %s'%(epoch)).center(200,'*')
+            train_data = tqdm.tqdm(train_data)
+        log = '\n' + ('Epoch %s' % (epoch)).center(200, '*')
         print(log)
         for step, (inputs, labels) in enumerate(train_data):
+            inputs=inputs.to(cfg.DEVICE)
+            labels=labels.to(cfg.DEVICE)
             global_step += 1
+            optimizer.zero_grad()
             outputs = model(inputs)
             loss = cfg.criterion(outputs, labels)
-            optimizer.zero_grad()
             loss.backward()
             losses.append(loss.item())
             optimizer.step()
@@ -270,7 +294,7 @@ def train(cfg):
                 )
                 torch.save(model.state_dict(),
                            os.path.join(cfg.WEIGHTS_SAVE_DIR, cfg.WEIGHTS_SAVE_NAME.format(**context)))
-            if global_step % val_interval==0 and global_step!=0:
+            if global_step % val_interval == 0 and global_step != 0:
                 val_res = val(cfg)
                 val_history.push(val_res)
                 log = '''\nStep:{global_step}\tValLoss:{val_loss:.4f}\tValAccuracy:{val_acc:.4f}\tValRecalls:{val_recalls}\tValPrecisions:{val_precisions}'''.format(
@@ -278,7 +302,7 @@ def train(cfg):
                     val_recalls=val_res['recalls'], val_precisions=val_res['precisions']
                 )
                 print(log)
-                context=dict(
+                context = dict(
                     tag=cfg.TAG,
                     epoch=epoch,
                     step=global_step,
@@ -288,9 +312,9 @@ def train(cfg):
 
                 if val_history.get('accuracy').last_is_best():
                     torch.save(model.state_dict(),
-                               os.path.join(cfg.WEIGHTS_SAVE_DIR,cfg.WEIGHTS_SAVE_BEST_NAME.format(**context)))
+                               os.path.join(cfg.WEIGHTS_SAVE_DIR, cfg.WEIGHTS_SAVE_BEST_NAME.format(**context)))
                     torch.save(model.state_dict(), cfg.WEIGHTS_SAVE_DIR + f'''/model_best.pkl''')
-                    print('New best accuracy: %.4f, model saved.'%(val_res['accuracy']))
+                    print('New best accuracy: %.4f, model saved.' % (val_res['accuracy']))
                 model.train()
         lr_scheduler.step()
         train_res = train_acc_metric.analyze()
@@ -303,5 +327,3 @@ def train(cfg):
         )
         print(log)
         torch.save(model.state_dict(), os.path.join(cfg.WEIGHTS_SAVE_DIR, 'model.pkl'))
-
-
