@@ -25,6 +25,7 @@ def make_trainval_config(
         VAL_DIR = cfg.VAL_DIR
         INPUT_W = cfg.INPUT_W
         INPUT_H = cfg.INPUT_H
+        USE_PRETRAINED = cfg.USE_PRETRAINED
         val_transform = EasyTransform([
             t.Resize(INPUT_SIZE[::-1]),
             t.SaveToDir(cfg.VISUALIZE_RESULT_DIR),
@@ -61,6 +62,7 @@ def get_base_config(train_dir='/home/ars/sda5/data/projects/烟分类/data/烟
         VAL_DIR=SelectDir(val_dir),
         GEN_CLASSES_FILE=VirtualField(default=True, title='生成类别文件'),
         USE_tqdm_TRAIN=True,
+        USE_PRETRAINED=True,
         BATCH_SIZE=64,
         MAX_EPOCHS=200,
         BALANCE_CLASSES=True,
